<ul class="flash-error">
	<li><span class='error_msg'><b>Warning:</b></span> The backup module of PostfixAdmin is poorly maintained and might contain bugs.
	<li>Please use <tt>mysqldump</tt> to get a reliable backup of your database.
	<li>&nbsp;
	<li>If you still trust this backup module, you can <a href="backup.php?download=1" class="button">download the database dump now</a>
</div>
