<?php

/****************************************************************************************

    Author ......... Florian Kimmerl
    Contact ........ info@spacekoeln.de
    Home Site ...... http://www.spacekoeln.de/
    Program ........ postfixadmin
    Version ........ 0.3-1.4
    Purpose ........ Allows you to change your postfixadmin settings within squirrelmail

 /**
 * index.php
 *
 * Copyright (c) 1999-2003 The SquirrelMail Project Team
 * Licensed under the GNU GPL. For full terms see the file COPYING.
 *
 * This file simply takes any attempt to view source files and sends those
 * people to the login screen. At this point no attempt is made to see if
 * the person is logged or not.
 *
 *
****************************************************************************************/

header("Location: ../../index.php");
